unit Reminder;

interface

uses
  SysUtils, Classes, ExtCtrls, Controls, DateUtils;
  
{$R Reminder.dcr}

type
  TReminderItem = class(TCollectionItem)
  private
    { Private declarations }
    FEnabled: Boolean;
    FText: String;
    FTag: Integer;
    procedure SetDate(const Value: String);
    function GetDate: String;
    function GetTime: String;
    procedure SetTime(const Value: String);
  protected
    { Protected declarations }
    FDate: TDate;
    FTime: TTime;
  public
    { Public declarations }
    constructor Create(Collection: TCollection); override;
    procedure Assign(Source: TPersistent); override;
  published
    { Published declarations }
    property Enabled: Boolean read FEnabled write FEnabled default True;
    property Date: String read GetDate write SetDate;
    property Time: String read GetTime write SetTime;
    property Text: String read FText write FText;
    property Tag: Integer read FTag write FTag default 0;
  end;

  TReminderNotify = procedure(Sender: TObject; ReminderItem: TReminderItem) of object;

  TReminderCollection = class(TCollection)
  private
    function GetItem(Index: Integer): TReminderItem;
    procedure SetItem(Index: Integer; const Value: TReminderItem);
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create;
    function Add: TReminderItem;
    property Items[Index: Integer]: TReminderItem read GetItem write SetItem; default;
  published
    { Published declarations }
  end;

  TReminder = class(TComponent)
  private
    { Private declarations }
    FTimer: TTimer;
    FReminders: TReminderCollection;
    FOnReminder: TReminderNotify;
    procedure SetEnabled(const Value: Boolean);
    procedure SetInterval(const Value: Cardinal);
    procedure SetReminders(const Value: TReminderCollection);
    function GetEnabled: Boolean;
    function GetInterval: Cardinal;
    procedure OnTimer(Sender: TObject);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Enabled: Boolean read GetEnabled write SetEnabled default True;
    property Interval: Cardinal read GetInterval write SetInterval default 1000;
    property Reminders: TReminderCollection read FReminders write SetReminders;
    property OnReminder: TReminderNotify read FOnReminder write FOnReminder;
  end;

procedure Register;

implementation

var
  FormatSettings: TFormatSettings;

procedure Register;
begin
  RegisterComponents('Samples', [TReminder]);
end;

{ TReminder }

constructor TReminder.Create(AOwner: TComponent);
begin
  inherited Create( AOwner );
  FReminders := TReminderCollection.Create;
  FTimer := TTimer.Create( Self );
  FTimer.OnTimer := OnTimer;
end;

destructor TReminder.Destroy;
begin
  FTimer.Free;
  FReminders.Free;
  inherited;
end;

function TReminder.GetEnabled: Boolean;
begin
  Result := FTimer.Enabled;
end;

function TReminder.GetInterval: Cardinal;
begin
  Result := FTimer.Interval;
end;

procedure TReminder.OnTimer(Sender: TObject);
var I: Integer;
    D, T: TDateTime;
    R: TReminderItem;
begin
  If Not(csDesigning in Self.ComponentState) and Enabled then
  begin
    D := SysUtils.Date;
    T := SysUtils.Time;
    For I := 0 to FReminders.Count - 1 do
    begin
      R := FReminders[I];
      If (( CompareDate(R.FDate, D) = -1 ) or
         ( (CompareDate(R.FDate, D) = 0) and (CompareTime(R.FTime, T) = -1) ))
         and R.Enabled then
      begin
        R.Enabled := False;
        If @FOnReminder <> Nil then FOnReminder(Self, R);
      end;
    end;
  end;
end;

procedure TReminder.SetEnabled(const Value: Boolean);
begin
  FTimer.Enabled := Value;
end;

procedure TReminder.SetInterval(const Value: Cardinal);
begin
  FTimer.Interval := Value;
end;

procedure TReminder.SetReminders(const Value: TReminderCollection);
begin
  FReminders.Assign( Value );
end;

{ TReminderItem }

constructor TReminderItem.Create(Collection: TCollection);
begin
  inherited Create( Collection );
  FDate := SysUtils.Date;
  FTime := SysUtils.Time;
  FTag := 0;
  FText := '';
  FEnabled := True;
end;

function TReminderItem.GetDate: String;
begin
  Result := DateToStr( FDate, FormatSettings );
end;

procedure TReminderItem.SetDate(const Value: String);
begin
  FDate := StrToDate( Value, FormatSettings );
end;

function TReminderItem.GetTime: String;
begin
  Result := TimeToStr( FTime, FormatSettings );
end;

procedure TReminderItem.SetTime(const Value: String);
begin
  FTime := StrToTime( Value, FormatSettings );
end;

procedure TReminderItem.Assign(Source: TPersistent);
begin
  If Source is TReminderItem then
  begin
    Date := TReminderItem(Source).Date;
    Time := TReminderItem(Source).Time;
    Text := TReminderItem(Source).Text;
    Tag := TReminderItem(Source).Tag;
  end;
end;

{ TReminderCollection }

function TReminderCollection.Add: TReminderItem;
begin
  Result := TReminderItem(inherited Add);
end;

constructor TReminderCollection.Create;
begin
  inherited Create( TReminderItem );
end;

function TReminderCollection.GetItem(Index: Integer): TReminderItem;
begin
  Result := TReminderItem(inherited GetItem( Index ));
end;

procedure TReminderCollection.SetItem(Index: Integer;
  const Value: TReminderItem);
begin
  inherited SetItem( Index, Value );
end;

initialization
  FormatSettings.DateSeparator := '.';
  FormatSettings.TimeSeparator := ':';
  FormatSettings.ShortDateFormat := 'dd.mm.yyyy.';
  FormatSettings.LongDateFormat := 'dd.mm.yyyy.';
  FormatSettings.ShortTimeFormat := 'hh:mm:ss';
  FormatSettings.LongTimeFormat := 'hh:mm:ss';
end.
